@@ -64,10 +64,19 @@ export const nodeSupportsSpawningChildProcess = semver.gte(
   '12.17.0'
 );
 export const nodeUsesNewHooksApi = semver.gte(process.version, '16.12.0');
-export const nodeSupportsImportAssertions = semver.gte(
-  process.version,
-  '17.1.0'
-);
+// 16.14.0: https://github.com/nodejs/node/blob/main/doc/changelogs/CHANGELOG_V16.md#notable-changes-4
+// 17.1.0: https://github.com/nodejs/node/blob/main/doc/changelogs/CHANGELOG_V17.md#2021-11-09-version-1710-current-targos
+export const nodeSupportsImportAssertions =
+  (semver.gte(process.version, '16.14.0') &&
+    semver.lt(process.version, '17.0.0')) ||
+  semver.gte(process.version, '17.1.0');
+// These versions do not require `--experimental-json-modules`
+// 16.15.0: https://github.com/nodejs/node/blob/main/doc/changelogs/CHANGELOG_V16.md#2022-04-26-version-16150-gallium-lts-danielleadams
+// 17.5.0: https://github.com/nodejs/node/blob/main/doc/changelogs/CHANGELOG_V17.md#2022-02-10-version-1750-current-ruyadorno
+export const nodeSupportsUnflaggedJsonImports =
+  (semver.gte(process.version, '16.15.0') &&
+    semver.lt(process.version, '17.0.0')) ||
+  semver.gte(process.version, '17.5.0');
 // Node 14.13.0 has a bug where it tries to lex CJS files to discover named exports *before*
 // we transform the code.
 // In other words, it tries to parse raw TS as CJS and balks at `export const foo =`, expecting to see `exports.foo =`
@@ -76,6 +85,7 @@ export const nodeSupportsImportingTransformedCjsFromEsm = semver.gte(
   process.version,
   '14.13.1'
 );
+export const tsSupportsResolveJsonModule = semver.gte(ts.version, '2.9.0');
 /** Supports tsconfig "extends" >= v3.2.0 */
 export const tsSupportsTsconfigInheritanceViaNodePackages = semver.gte(
   ts.version,
@@ -88,6 +98,7 @@ export const tsSupportsStableNodeNextNode16 =
   ts.version.startsWith('4.7.') || semver.gte(ts.version, '4.7.0');
 // TS 4.5 is first version to understand .cts, .mts, .cjs, and .mjs extensions
 export const tsSupportsMtsCtsExtensions = semver.gte(ts.version, '4.5.0');
+export const tsSupportsImportAssertions = semver.gte(ts.version, '4.5.0');
 //#endregion
 
 export const xfs = new NodeFS(fs);
