@@ -1,19 +1,22 @@
-import { _test } from './testlib';
+import { context, ExecutionContext } from './testlib';
 import * as expect from 'expect';
 import { join, resolve, sep as pathSep } from 'path';
 import { tmpdir } from 'os';
 import semver = require('semver');
 import {
   BIN_PATH_JS,
+  CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG,
   nodeSupportsEsmHooks,
+  nodeSupportsSpawningChildProcess,
   ts,
+  tsSupportsMtsCtsExtensions,
   tsSupportsShowConfig,
+  tsSupportsStableNodeNextNode16,
   tsSupportsTsconfigInheritanceViaNodePackages,
 } from './helpers';
 import { lstatSync, mkdtempSync } from 'fs';
 import { npath } from '@yarnpkg/fslib';
 import type _createRequire from 'create-require';
-import { pathToFileURL } from 'url';
 import { createExec } from './exec-helpers';
 import {
   BIN_CWD_PATH,
@@ -25,18 +28,18 @@ import {
   testsDirRequire,
   tsNodeTypes,
   xfs,
-  contextTsNodeUnderTest,
+  ctxTsNode,
   CMD_TS_NODE_WITH_PROJECT_FLAG,
   CMD_TS_NODE_WITHOUT_PROJECT_FLAG,
   CMD_ESM_LOADER_WITHOUT_PROJECT,
-  EXPERIMENTAL_MODULES_FLAG,
 } from './helpers';
+import type { CreateOptions } from '..';
 
 const exec = createExec({
   cwd: TEST_DIR,
 });
 
-const test = _test.context(contextTsNodeUnderTest);
+const test = context(ctxTsNode);
 
 test.suite('ts-node', (test) => {
   test('should export the correct version', (t) => {
@@ -70,11 +73,13 @@ test.suite('ts-node', (test) => {
     testsDirRequire.resolve('ts-node/register/transpile-only');
     testsDirRequire.resolve('ts-node/register/type-check');
 
-    // `node --loader ts-node/esm`
-    testsDirRequire.resolve('ts-node/esm');
-    testsDirRequire.resolve('ts-node/esm.mjs');
-    testsDirRequire.resolve('ts-node/esm/transpile-only');
-    testsDirRequire.resolve('ts-node/esm/transpile-only.mjs');
+    if (semver.gte(process.version, '12.17.0')) {
+      // `node --loader ts-node/esm`
+      testsDirRequire.resolve('ts-node/esm');
+      testsDirRequire.resolve('ts-node/esm.mjs');
+      testsDirRequire.resolve('ts-node/esm/transpile-only');
+      testsDirRequire.resolve('ts-node/esm/transpile-only.mjs');
+    }
 
     testsDirRequire.resolve('ts-node/transpilers/swc');
     testsDirRequire.resolve('ts-node/transpilers/swc-experimental');
@@ -143,7 +148,10 @@ test.suite('ts-node', (test) => {
 
     test('should execute cli with absolute path', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} "${join(TEST_DIR, 'hello-world')}"`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} "${join(
+          TEST_DIR,
+          'hello-world'
+        )}"`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('Hello, world!\n');
@@ -157,26 +165,18 @@ test.suite('ts-node', (test) => {
       expect(stdout).toBe('example\n');
     });
 
-    test('should provide registered information globally', async () => {
+    test("should expose ts-node Service as a symbol property on Node's `process` object", async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} env`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} env`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('object\n');
     });
 
-    test('should provide registered information on register', async () => {
-      const { err, stdout } = await exec(`node -r ts-node/register env.ts`, {
-        cwd: TEST_DIR,
-      });
-      expect(err).toBe(null);
-      expect(stdout).toBe('object\n');
-    });
-
     test('should allow js', async () => {
       const { err, stdout } = await exec(
         [
-          CMD_TS_NODE_WITH_PROJECT_FLAG,
+          CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG,
           '-O "{\\"allowJs\\":true}"',
           '-pe "import { main } from \'./allow-js/run\';main()"',
         ].join(' ')
@@ -188,7 +188,7 @@ test.suite('ts-node', (test) => {
     test('should include jsx when `allow-js` true', async () => {
       const { err, stdout } = await exec(
         [
-          CMD_TS_NODE_WITH_PROJECT_FLAG,
+          CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG,
           '-O "{\\"allowJs\\":true}"',
           '-pe "import { Foo2 } from \'./allow-js/with-jsx\'; Foo2.sayHi()"',
         ].join(' ')
@@ -197,9 +197,42 @@ test.suite('ts-node', (test) => {
       expect(stdout).toBe('hello world\n');
     });
 
+    test.suite('should support cts when module = CommonJS', (test) => {
+      test.runIf(tsSupportsMtsCtsExtensions);
+      test('test', async (t) => {
+        const { err, stdout } = await exec(
+          [
+            CMD_TS_NODE_WITHOUT_PROJECT_FLAG,
+            '-pe "import { main } from \'./index.cjs\';main()"',
+          ].join(' '),
+          {
+            cwd: join(TEST_DIR, 'ts45-ext/ext-cts'),
+          }
+        );
+        expect(err).toBe(null);
+        expect(stdout).toBe('hello world\n');
+      });
+    });
+
+    test.suite('should support mts when module = ESNext', (test) => {
+      test.runIf(
+        nodeSupportsSpawningChildProcess && tsSupportsMtsCtsExtensions
+      );
+      test('test', async () => {
+        const { err, stdout } = await exec(
+          [CMD_TS_NODE_WITHOUT_PROJECT_FLAG, './entrypoint.mjs'].join(' '),
+          {
+            cwd: join(TEST_DIR, 'ts45-ext/ext-mts'),
+          }
+        );
+        expect(err).toBe(null);
+        expect(stdout).toBe('hello world\n');
+      });
+    });
+
     test('should eval code', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} -e "import * as m from './module';console.log(m.example('test'))"`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} -e "import * as m from './module';console.log(m.example('test'))"`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('TEST\n');
@@ -207,13 +240,13 @@ test.suite('ts-node', (test) => {
 
     test('should import empty files', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} -e "import './empty'"`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} -e "import './empty'"`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('');
     });
 
-    test('should throw errors', async () => {
+    test('should throw typechecking errors', async () => {
       const { err } = await exec(
         `${CMD_TS_NODE_WITH_PROJECT_FLAG} -e "import * as m from './module';console.log(m.example(123))"`
       );
@@ -377,7 +410,7 @@ test.suite('ts-node', (test) => {
     }
 
     test('should pipe into `ts-node` and evaluate', async () => {
-      const execPromise = exec(CMD_TS_NODE_WITH_PROJECT_FLAG);
+      const execPromise = exec(CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG);
       execPromise.child.stdin!.end("console.log('hello')");
       const { err, stdout } = await execPromise;
       expect(err).toBe(null);
@@ -385,7 +418,9 @@ test.suite('ts-node', (test) => {
     });
 
     test('should pipe into `ts-node`', async () => {
-      const execPromise = exec(`${CMD_TS_NODE_WITH_PROJECT_FLAG} -p`);
+      const execPromise = exec(
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} -p`
+      );
       execPromise.child.stdin!.end('true');
       const { err, stdout } = await execPromise;
       expect(err).toBe(null);
@@ -404,7 +439,7 @@ test.suite('ts-node', (test) => {
 
     test('should support require flags', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} -r ./hello-world -pe "console.log('success')"`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} -r ./hello-world -pe "console.log('success')"`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('Hello, world!\nsuccess\nundefined\n');
@@ -412,7 +447,7 @@ test.suite('ts-node', (test) => {
 
     test('should support require from node modules', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} -r typescript -e "console.log('success')"`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} -r typescript -e "console.log('success')"`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('success\n');
@@ -458,28 +493,17 @@ test.suite('ts-node', (test) => {
       );
     });
 
-    test('should preserve `ts-node` context with child process', async () => {
-      const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} child-process`
-      );
-      expect(err).toBe(null);
-      expect(stdout).toBe('Hello, world!\n');
-    });
-
     test('should import js before ts by default', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} import-order/compiled`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} import-order/compiled`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('Hello, JavaScript!\n');
     });
 
-    const preferTsExtsEntrypoint = semver.gte(process.version, '12.0.0')
-      ? 'import-order/compiled'
-      : 'import-order/require-compiled';
     test('should import ts before js when --prefer-ts-exts flag is present', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} --prefer-ts-exts ${preferTsExtsEntrypoint}`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} --prefer-ts-exts import-order/compiled`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('Hello, TypeScript!\n');
@@ -487,7 +511,7 @@ test.suite('ts-node', (test) => {
 
     test('should import ts before js when TS_NODE_PREFER_TS_EXTS env is present', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} ${preferTsExtsEntrypoint}`,
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} import-order/compiled`,
         {
           env: { ...process.env, TS_NODE_PREFER_TS_EXTS: 'true' },
         }
@@ -498,7 +522,7 @@ test.suite('ts-node', (test) => {
 
     test('should ignore .d.ts files', async () => {
       const { err, stdout } = await exec(
-        `${CMD_TS_NODE_WITH_PROJECT_FLAG} import-order/importer`
+        `${CMD_TS_NODE_WITH_PROJECT_TRANSPILE_ONLY_FLAG} import-order/importer`
       );
       expect(err).toBe(null);
       expect(stdout).toBe('Hello, World!\n');
@@ -538,63 +562,60 @@ test.suite('ts-node', (test) => {
       });
     });
 
-    if (semver.gte(ts.version, '2.7.0')) {
-      test('should locate tsconfig relative to entry-point by default', async () => {
-        const { err, stdout } = await exec(`${BIN_PATH} ../a/index`, {
+    test('should locate tsconfig relative to entry-point by default', async () => {
+      const { err, stdout } = await exec(`${BIN_PATH} ../a/index`, {
+        cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
+      });
+      expect(err).toBe(null);
+      expect(stdout).toMatch(/plugin-a/);
+    });
+    test('should locate tsconfig relative to entry-point via ts-node-script', async () => {
+      const { err, stdout } = await exec(`${BIN_SCRIPT_PATH} ../a/index`, {
+        cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
+      });
+      expect(err).toBe(null);
+      expect(stdout).toMatch(/plugin-a/);
+    });
+    test('should locate tsconfig relative to entry-point with --script-mode', async () => {
+      const { err, stdout } = await exec(
+        `${BIN_PATH} --script-mode ../a/index`,
+        {
           cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
-        });
-        expect(err).toBe(null);
-        expect(stdout).toMatch(/plugin-a/);
-      });
-      test('should locate tsconfig relative to entry-point via ts-node-script', async () => {
-        const { err, stdout } = await exec(`${BIN_SCRIPT_PATH} ../a/index`, {
-          cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
-        });
-        expect(err).toBe(null);
-        expect(stdout).toMatch(/plugin-a/);
-      });
-      test('should locate tsconfig relative to entry-point with --script-mode', async () => {
-        const { err, stdout } = await exec(
-          `${BIN_PATH} --script-mode ../a/index`,
-          {
-            cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
-          }
-        );
-        expect(err).toBe(null);
-        expect(stdout).toMatch(/plugin-a/);
-      });
-      test('should locate tsconfig relative to cwd via ts-node-cwd', async () => {
-        const { err, stdout } = await exec(`${BIN_CWD_PATH} ../a/index`, {
-          cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
-        });
-        expect(err).toBe(null);
-        expect(stdout).toMatch(/plugin-b/);
-      });
-      test('should locate tsconfig relative to cwd in --cwd-mode', async () => {
-        const { err, stdout } = await exec(
-          `${BIN_PATH} --cwd-mode ../a/index`,
-          { cwd: join(TEST_DIR, 'cwd-and-script-mode/b') }
-        );
-        expect(err).toBe(null);
-        expect(stdout).toMatch(/plugin-b/);
-      });
-      test('should locate tsconfig relative to realpath, not symlink, when entrypoint is a symlink', async (t) => {
-        if (
-          lstatSync(
-            join(TEST_DIR, 'main-realpath/symlink/symlink.tsx')
-          ).isSymbolicLink()
-        ) {
-          const { err, stdout } = await exec(
-            `${BIN_PATH} main-realpath/symlink/symlink.tsx`
-          );
-          expect(err).toBe(null);
-          expect(stdout).toBe('');
-        } else {
-          t.log('Skipping');
-          return;
         }
+      );
+      expect(err).toBe(null);
+      expect(stdout).toMatch(/plugin-a/);
+    });
+    test('should locate tsconfig relative to cwd via ts-node-cwd', async () => {
+      const { err, stdout } = await exec(`${BIN_CWD_PATH} ../a/index`, {
+        cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
       });
-    }
+      expect(err).toBe(null);
+      expect(stdout).toMatch(/plugin-b/);
+    });
+    test('should locate tsconfig relative to cwd in --cwd-mode', async () => {
+      const { err, stdout } = await exec(`${BIN_PATH} --cwd-mode ../a/index`, {
+        cwd: join(TEST_DIR, 'cwd-and-script-mode/b'),
+      });
+      expect(err).toBe(null);
+      expect(stdout).toMatch(/plugin-b/);
+    });
+    test('should locate tsconfig relative to realpath, not symlink, when entrypoint is a symlink', async (t) => {
+      if (
+        lstatSync(
+          join(TEST_DIR, 'main-realpath/symlink/symlink.tsx')
+        ).isSymbolicLink()
+      ) {
+        const { err, stdout } = await exec(
+          `${BIN_PATH} main-realpath/symlink/symlink.tsx`
+        );
+        expect(err).toBe(null);
+        expect(stdout).toBe('');
+      } else {
+        t.log('Skipping');
+        return;
+      }
+    });
 
     test.suite('should read ts-node options from tsconfig.json', (test) => {
       const BIN_EXEC = `"${BIN_PATH}" --project tsconfig-options/tsconfig.json`;
@@ -725,8 +746,8 @@ test.suite('ts-node', (test) => {
 
     test.suite(
       'should use implicit @tsconfig/bases config when one is not loaded from disk',
-      (_test) => {
-        const test = _test.context(async (t) => ({
+      ({ context }) => {
+        const test = context(async (t) => ({
           tempDir: mkdtempSync(join(tmpdir(), 'ts-node-spec')),
         }));
         if (
@@ -957,8 +978,8 @@ test.suite('ts-node', (test) => {
     });
   });
 
-  test.suite('create', (_test) => {
-    const test = _test.context(async (t) => {
+  test.suite('create', ({ context }) => {
+    const test = context(async (t) => {
       return {
         service: t.context.tsNodeUnderTest.create({
           compilerOptions: { target: 'es5' },
@@ -997,64 +1018,76 @@ test.suite('ts-node', (test) => {
   });
 
   test.suite('issue #1098', (test) => {
-    function testIgnored(
-      ignored: tsNodeTypes.Service['ignored'],
-      allowed: string[],
-      disallowed: string[]
+    function testAllowedExtensions(
+      t: ExecutionContext<ctxTsNode.Ctx>,
+      compilerOptions: CreateOptions['compilerOptions'],
+      allowed: string[]
     ) {
+      const disallowed = allExtensions.filter((ext) => !allowed.includes(ext));
+      const { ignored } = t.context.tsNodeUnderTest.create({
+        compilerOptions,
+        skipProject: true,
+      });
       for (const ext of allowed) {
-        // should accept ${ext} files
+        t.log(`Testing that ${ext} files are allowed`);
         expect(ignored(join(DIST_DIR, `index${ext}`))).toBe(false);
       }
       for (const ext of disallowed) {
-        // should ignore ${ext} files
+        t.log(`Testing that ${ext} files are ignored`);
         expect(ignored(join(DIST_DIR, `index${ext}`))).toBe(true);
       }
     }
 
+    const allExtensions = [
+      '.ts',
+      '.js',
+      '.d.ts',
+      '.mts',
+      '.cts',
+      '.d.mts',
+      '.d.cts',
+      '.mjs',
+      '.cjs',
+      '.tsx',
+      '.jsx',
+      '.xyz',
+      '',
+    ];
+    const mtsCts = tsSupportsMtsCtsExtensions
+      ? ['.mts', '.cts', '.d.mts', '.d.cts']
+      : [];
+    const mjsCjs = tsSupportsMtsCtsExtensions ? ['.mjs', '.cjs'] : [];
+
     test('correctly filters file extensions from the compiler when allowJs=false and jsx=false', (t) => {
-      const { ignored } = t.context.tsNodeUnderTest.create({
-        compilerOptions: {},
-        skipProject: true,
-      });
-      testIgnored(
-        ignored,
-        ['.ts', '.d.ts'],
-        ['.js', '.tsx', '.jsx', '.mjs', '.cjs', '.xyz', '']
-      );
+      testAllowedExtensions(t, {}, ['.ts', '.d.ts', ...mtsCts]);
     });
     test('correctly filters file extensions from the compiler when allowJs=true and jsx=false', (t) => {
-      const { ignored } = t.context.tsNodeUnderTest.create({
-        compilerOptions: { allowJs: true },
-        skipProject: true,
-      });
-      testIgnored(
-        ignored,
-        ['.ts', '.js', '.d.ts'],
-        ['.tsx', '.jsx', '.mjs', '.cjs', '.xyz', '']
-      );
+      testAllowedExtensions(t, { allowJs: true }, [
+        '.ts',
+        '.js',
+        '.d.ts',
+        ...mtsCts,
+        ...mjsCjs,
+      ]);
     });
     test('correctly filters file extensions from the compiler when allowJs=false and jsx=true', (t) => {
-      const { ignored } = t.context.tsNodeUnderTest.create({
-        compilerOptions: { allowJs: false, jsx: 'preserve' },
-        skipProject: true,
-      });
-      testIgnored(
-        ignored,
-        ['.ts', '.tsx', '.d.ts'],
-        ['.js', '.jsx', '.mjs', '.cjs', '.xyz', '']
-      );
+      testAllowedExtensions(t, { allowJs: false, jsx: 'preserve' }, [
+        '.ts',
+        '.tsx',
+        '.d.ts',
+        ...mtsCts,
+      ]);
     });
     test('correctly filters file extensions from the compiler when allowJs=true and jsx=true', (t) => {
-      const { ignored } = t.context.tsNodeUnderTest.create({
-        compilerOptions: { allowJs: true, jsx: 'preserve' },
-        skipProject: true,
-      });
-      testIgnored(
-        ignored,
-        ['.ts', '.tsx', '.js', '.jsx', '.d.ts'],
-        ['.mjs', '.cjs', '.xyz', '']
-      );
+      testAllowedExtensions(t, { allowJs: true, jsx: 'preserve' }, [
+        '.ts',
+        '.tsx',
+        '.js',
+        '.jsx',
+        '.d.ts',
+        ...mtsCts,
+        ...mjsCjs,
+      ]);
     });
   });
 });
@@ -1135,7 +1168,11 @@ test('Detect when typescript adds new ModuleKind values; flag as a failure so we
     expect(ts.ModuleKind[99]).toBeUndefined();
   }
   check(7, 'ES2022', false);
-  check(100, 'Node12', false);
+  if (tsSupportsStableNodeNextNode16) {
+    check(100, 'Node16', true);
+  } else {
+    check(100, 'Node12', false);
+  }
   check(199, 'NodeNext', false);
   const actualKeys = Object.keys(ts.ModuleKind);
   actualKeys.sort();
